pub struct ExpectArray { /* private fields */ }Implementations
Sourceimpl ExpectArray
impl ExpectArray
pub fn empty(self) -> ExpectArray
pub fn not_empty(self) -> ExpectArray
pub fn len(self, len: usize) -> ExpectArray
pub fn min_len(self, min_len: usize) -> ExpectArray
pub fn max_len(self, max_len: usize) -> ExpectArray
pub fn contains<I, V>(self, expected_values: I) -> ExpectArray
Sourcepub fn eq_unordered<I, V>(self, expected_values: I) -> ExpectArray
pub fn eq_unordered<I, V>(self, expected_values: I) -> ExpectArray
Expects all values in the array match the expected values in some order.
This can be an exact value, or an ExpectOp.
The lengths of the arrays must be equal.
Sourcepub fn all<V>(self, expected: V) -> ExpectArray
pub fn all<V>(self, expected: V) -> ExpectArray
Expects all values in the array match the expected value.
This can be an exact value, or an ExpectOp.
Note an empty array will match this.
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/users")
.await
.assert_json(&expect_json::array().all(
json!({
"name": expect_json::string().not_empty(),
"email": expect_json::email(),
})
));Sourcepub fn all_contains<V>(self, expected: V) -> ExpectArray
pub fn all_contains<V>(self, expected: V) -> ExpectArray
Expects all values to be an object, containing the expected value.
Example Usage
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/users")
.await
.assert_json(&expect_json::array().all_contains(json!({
"name": expect_json::string().not_empty(),
"email": expect_json::email(),
})));Equivalent Usage
This is shorthand for using ExpectArray::all and ExpectObject::contains, like so:
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/users")
.await
.assert_json(&expect_json::array().all(
expect_json::object().contains(json!({
"name": expect_json::string().not_empty(),
"email": expect_json::email(),
}))
));Sourcepub fn unique(self) -> ExpectArray
pub fn unique(self) -> ExpectArray
Expects all values in the array are unique. No duplicates.
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/users")
.await
.assert_json(&json!({
// expect an array of unique UUIDs
"user_ids": expect_json::array()
.all(expect_json::uuid())
.unique(),
}));Trait Implementations
Sourceimpl Clone for ExpectArray
impl Clone for ExpectArray
Sourcefn clone(&self) -> ExpectArray
fn clone(&self) -> ExpectArray
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Debug for ExpectArray
impl Debug for ExpectArray
Sourceimpl Default for ExpectArray
impl Default for ExpectArray
Sourcefn default() -> ExpectArray
fn default() -> ExpectArray
Sourceimpl<'de> Deserialize<'de> for ExpectArray
impl<'de> Deserialize<'de> for ExpectArray
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<ExpectArray, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<ExpectArray, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Sourceimpl ExpectOp for ExpectArray
impl ExpectOp for ExpectArray
fn on_array( &self, context: &mut Context<'_>, received: &[Value], ) -> Result<(), ExpectOpError>
Sourcefn debug_supported_types(&self) -> &'static [JsonType]
fn debug_supported_types(&self) -> &'static [JsonType]
fn on_any( &self, context: &mut Context<'_>, received: &Value, ) -> Result<(), ExpectOpError>
fn on_null(&self, context: &mut Context<'_>) -> Result<(), ExpectOpError>
fn on_f64( &self, context: &mut Context<'_>, received: f64, ) -> Result<(), ExpectOpError>
fn on_u64( &self, context: &mut Context<'_>, received: u64, ) -> Result<(), ExpectOpError>
fn on_i64( &self, context: &mut Context<'_>, received: i64, ) -> Result<(), ExpectOpError>
fn on_boolean( &self, context: &mut Context<'_>, received: bool, ) -> Result<(), ExpectOpError>
fn on_string( &self, context: &mut Context<'_>, received: &str, ) -> Result<(), ExpectOpError>
fn on_object( &self, context: &mut Context<'_>, received: &Map<String, Value>, ) -> Result<(), ExpectOpError>
Sourceimpl PartialEq for ExpectArray
impl PartialEq for ExpectArray
Sourceimpl Serialize for ExpectArray
impl Serialize for ExpectArray
Sourcefn serialize<S>(
&self,
serializer: S,
) -> Result<<S as Serializer>::Ok, <S as Serializer>::Error>where
S: Serializer,
fn serialize<S>(
&self,
serializer: S,
) -> Result<<S as Serializer>::Ok, <S as Serializer>::Error>where
S: Serializer,
impl StructuralPartialEq for ExpectArray