ExpectIsoDateTime

Struct ExpectIsoDateTime 

pub struct ExpectIsoDateTime { /* private fields */ }
Expects an ISO 8601 date time string.

By default this accepts any timezone.

use std::time::Duration;
use axum_test::expect_json;

let server = TestServer::new(Router::new());

server.get(&"/latest-comment")
    .await
    .assert_json(&json!({
        "comment": "My example comment",
        "created_at": expect_json::iso_date_time(),
    }));

Implementations

Source

impl ExpectIsoDateTime

pub fn utc(self) -> ExpectIsoDateTime

By default, IsoDateTime expects all date times to be in UTC.

This method relaxes this constraint, and will accept date times in any timezone.

use std::time::Duration;
use axum_test::expect_json;

let server = TestServer::new(Router::new());

server.get(&"/latest-comment")
    .await
    .assert_json(&json!({
        "comment": "My example comment",

        // Users time may have any timezone
        "users_created_at": expect_json::iso_date_time().utc(),
    }));
pub fn within_past(self, duration: Duration) -> ExpectIsoDateTime

Expects the date time to be within a past duration, up to the current time.

The constraint will fail when:

  • the datetime is further in the past than the given duration,
  • or ahead of the current time.
use std::time::Duration;
use axum_test::expect_json;

let server = TestServer::new(Router::new());

server.get(&"/latest-comment")
    .await
    .assert_json(&json!({
        "comment": "My example comment",

        // Expect it was updated in the last minute
        "updated_at": expect_json::iso_date_time()
            .within_past(Duration::from_secs(60)),
    }));
pub fn within_future(self, duration: Duration) -> ExpectIsoDateTime

Expects the date time to be within the current time, and up to a future duration.

The constraint will fail when:

  • the datetime is further ahead than the given duration,
  • or behind the current time.
use std::time::Duration;
use axum_test::expect_json;

let server = TestServer::new(Router::new());

server.get(&"/latest-comment")
    .await
    .assert_json(&json!({
        "comment": "My example comment",

        // Expect it also expires in the next minute
        "expires_at": expect_json::iso_date_time()
            .within_future(Duration::from_secs(60)),
    }));