pub struct ExpectIsoDateTime { /* private fields */ }Expand description
Expects an ISO 8601 date time string.
By default this accepts any timezone.
use std::time::Duration;
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/latest-comment")
.await
.assert_json(&json!({
"comment": "My example comment",
"created_at": expect_json::iso_date_time(),
}));Implementations
Sourceimpl ExpectIsoDateTime
impl ExpectIsoDateTime
Sourcepub fn utc(self) -> ExpectIsoDateTime
pub fn utc(self) -> ExpectIsoDateTime
By default, IsoDateTime expects all date times to be in UTC.
This method relaxes this constraint, and will accept date times in any timezone.
use std::time::Duration;
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/latest-comment")
.await
.assert_json(&json!({
"comment": "My example comment",
// Users time may have any timezone
"users_created_at": expect_json::iso_date_time().utc(),
}));Sourcepub fn within_past(self, duration: Duration) -> ExpectIsoDateTime
pub fn within_past(self, duration: Duration) -> ExpectIsoDateTime
Expects the date time to be within a past duration, up to the current time.
The constraint will fail when:
- the datetime is further in the past than the given duration,
- or ahead of the current time.
use std::time::Duration;
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/latest-comment")
.await
.assert_json(&json!({
"comment": "My example comment",
// Expect it was updated in the last minute
"updated_at": expect_json::iso_date_time()
.within_past(Duration::from_secs(60)),
}));Sourcepub fn within_future(self, duration: Duration) -> ExpectIsoDateTime
pub fn within_future(self, duration: Duration) -> ExpectIsoDateTime
Expects the date time to be within the current time, and up to a future duration.
The constraint will fail when:
- the datetime is further ahead than the given duration,
- or behind the current time.
use std::time::Duration;
use axum_test::expect_json;
let server = TestServer::new(Router::new());
server.get(&"/latest-comment")
.await
.assert_json(&json!({
"comment": "My example comment",
// Expect it also expires in the next minute
"expires_at": expect_json::iso_date_time()
.within_future(Duration::from_secs(60)),
}));Trait Implementations
Sourceimpl Clone for ExpectIsoDateTime
impl Clone for ExpectIsoDateTime
Sourcefn clone(&self) -> ExpectIsoDateTime
fn clone(&self) -> ExpectIsoDateTime
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
source. Read moreSourceimpl Debug for ExpectIsoDateTime
impl Debug for ExpectIsoDateTime
Sourceimpl Default for ExpectIsoDateTime
impl Default for ExpectIsoDateTime
Sourcefn default() -> ExpectIsoDateTime
fn default() -> ExpectIsoDateTime
Sourceimpl<'de> Deserialize<'de> for ExpectIsoDateTime
impl<'de> Deserialize<'de> for ExpectIsoDateTime
Sourcefn deserialize<__D>(
__deserializer: __D,
) -> Result<ExpectIsoDateTime, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
fn deserialize<__D>(
__deserializer: __D,
) -> Result<ExpectIsoDateTime, <__D as Deserializer<'de>>::Error>where
__D: Deserializer<'de>,
Sourceimpl ExpectOp for ExpectIsoDateTime
impl ExpectOp for ExpectIsoDateTime
fn on_string( &self, context: &mut Context<'_>, received: &str, ) -> Result<(), ExpectOpError>
Sourcefn debug_supported_types(&self) -> &'static [JsonType]
fn debug_supported_types(&self) -> &'static [JsonType]
fn on_any( &self, context: &mut Context<'_>, received: &Value, ) -> Result<(), ExpectOpError>
fn on_null(&self, context: &mut Context<'_>) -> Result<(), ExpectOpError>
fn on_f64( &self, context: &mut Context<'_>, received: f64, ) -> Result<(), ExpectOpError>
fn on_u64( &self, context: &mut Context<'_>, received: u64, ) -> Result<(), ExpectOpError>
fn on_i64( &self, context: &mut Context<'_>, received: i64, ) -> Result<(), ExpectOpError>
fn on_boolean( &self, context: &mut Context<'_>, received: bool, ) -> Result<(), ExpectOpError>
fn on_array( &self, context: &mut Context<'_>, received: &[Value], ) -> Result<(), ExpectOpError>
fn on_object( &self, context: &mut Context<'_>, received: &Map<String, Value>, ) -> Result<(), ExpectOpError>
Sourceimpl PartialEq for ExpectIsoDateTime
impl PartialEq for ExpectIsoDateTime
Sourceimpl Serialize for ExpectIsoDateTime
impl Serialize for ExpectIsoDateTime
Sourcefn serialize<S>(
&self,
serializer: S,
) -> Result<<S as Serializer>::Ok, <S as Serializer>::Error>where
S: Serializer,
fn serialize<S>(
&self,
serializer: S,
) -> Result<<S as Serializer>::Ok, <S as Serializer>::Error>where
S: Serializer,
impl StructuralPartialEq for ExpectIsoDateTime